package pah;

import com.jme3.math.Vector2f;
import cz.agents.alite.creator.Creator;
import cz.agents.alite.simulation.Simulation;
import cz.agents.alite.vis.VisManager;
import cz.agents.alite.vis.layer.common.*;
import cz.agents.alite.vis3d.Vis3dManager;
import cz.agents.alite.vis3d.layer.Logo3DLayer;
import java.awt.Color;
import java.io.File;
import java.io.FileInputStream;
import java.util.Properties;
import java.util.Random;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import tacv.universe.AgentScoutUniverse;
import tacv.universe.environment.AgentScoutEnvironment;
import tacv.util.Point;
import tacv.vis.*;
import tacv.vis3d.layer.AS2Setting3DLayer;
import tacv.vis3d.layer.Car3DLayer;
import tacv.vis3d.layer.Skybox3DLayer;
import tacv.vis3d.layer.Village3DLayer;

public abstract class AbstractPahCarCreator implements Creator {

    private static final Logger LOGGER = Logger.getLogger(AbstractPahCarCreator.class);
    private static String overrideLogFileName = null;
    protected AgentScoutUniverse universe;
    protected final String villageMapXMLFile = "resources/map/village.xml";

    @Override
    public void init(String[] args) {
        if (args.length > 1) {
            overrideLogFileName = args[1];
        }

        Properties prop = new Properties();
        try {
            prop.load(new FileInputStream("resources" + File.separator + "log4j" + File.separator + "log4j.properties"));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        if (overrideLogFileName != null) {
            prop.setProperty("log4j.appender.R.File", overrideLogFileName);
        }
        PropertyConfigurator.configure(prop);
    }

    @Override
    public void create() {

        LOGGER.info(">>> UNIVERSE CREATION");
        universe = new AgentScoutUniverse();

        createSimulation();

        createEnvironment();

        createVisualization();

        createEntities();

        createOverlayVisualization();

        create3DVisualization();

        LOGGER.info(">>> UNIVERSE START");
        universe.start();
    }

    protected void createSimulation() {
        LOGGER.info(">>> SIMULATION CREATION");
        Simulation simulation = new Simulation();
        simulation.setPrintouts(10000);
        simulation.setSimulationSpeed(0.2);
        universe.setSimulation(simulation);
    }

    protected void createEnvironment() {
        LOGGER.info(">>> ENVIRONMENT CREATION");
        AgentScoutEnvironment environment = new AgentScoutEnvironment(universe, new File(villageMapXMLFile));
        environment.setRandom(new Random(0));
        universe.setEnvironment(environment);
//        WorldInfo.getInstance().initBulletPhysics();
    }

    protected void createVisualization() {
        LOGGER.info(">>> VISUALIZATION CREATION");
        VisManager.setInitParam("AgentScout Operator", 1024, 768);
        VisManager.init();

        // background
        VisManager.registerLayer(ColorLayer.create(Color.GRAY));

        // static
        VisManager.registerLayer(GraphLayer.create(universe.getEnvironment().getMap().getStreetGraph(), Color.BLACK, Color.BLUE, 1, 5));
        VisManager.registerLayer(BuildingLayer.create(universe.getEnvironment().getMap(), new Color(191, 162, 136), 1));

        // dynamic
        VisManager.registerLayer(ZoneLayer.create(universe.getEnvironment().getWorldInfo().getCollisionZone()));
        VisManager.registerLayer(CarLayer.create(universe.getEnvironment().getCarStorage()));

        VisManager.registerLayer(BubbleLayer.create());
    }

    protected void createEntities() {
        LOGGER.info(">>> ENTITIES CREATION");

        Point randomMapPoint = (Point) universe.getEnvironment().getMap().getStreetGraph().getRandomNode().clone();

        double height21 = universe.getEnvironment().getWorldInfo().getAltitude((float) randomMapPoint.x, (float) randomMapPoint.y);
        randomMapPoint.z = height21 + 2;
        System.out.println(randomMapPoint);
//        universe.addCarRandomMoving("car", randomMapPoint);
        universe.addCarAgent(getCarAgent(), randomMapPoint);

    }

    public abstract PahCarAgent getCarAgent();

    protected void createOverlayVisualization() {
        LOGGER.info(">>> OVERLAY VISUALIZATION CREATION");

        VisManager.registerLayer(SimulationControlLayer.create(universe.getSimulation()));
        VisManager.registerLayer(FpsLayer.create());
        VisManager.registerLayer(VisInfoLayer.create());
        VisManager.registerLayer(LogoLayer.create(new File("resources/img/atg_blue.png")));
        VisManager.registerLayer(HelpLayer.create());
    }

    protected void create3DVisualization() {

        Vis3dManager.setInitParam("AgentScout2 World", 1024, 768);

        Vis3dManager.registerSettingLayer(AS2Setting3DLayer.create());
        Vis3dManager.registerLayer(Logo3DLayer.create("img/atg_blue.png", new Vector2f(128, 64), new Vector2f(20, 0)));

        Vis3dManager.registerLayer(Skybox3DLayer.create());
        Vis3dManager.registerLayer(Village3DLayer.create());

        Vis3dManager.registerLayer(Car3DLayer.create(universe.getEnvironment().getCarStorage(), universe.getEnvironment().getWorldInfo().getTerrain()));
        Vis3dManager.startVis();
    }
}
