package tacv.universe.environment.sensor;

import com.jme3.math.Vector3f;
import cz.agents.alite.common.entity.Entity;
import cz.agents.alite.common.event.Event;
import cz.agents.alite.environment.eventbased.EventBasedSensor;
import java.util.HashMap;
import java.util.Map;
import tacv.universe.environment.AgentScoutEnvironment;
import tacv.universe.environment.sensor.callback.PositionChangeCallback;
import tacv.universe.environment.sensor.callback.WaypointReachedCallback;
import tacv.universe.environment.storage.CarStorage;
import tacv.util.Point;
import tacv.util.TacEventType;

public class CarSensor extends EventBasedSensor {

    private final static long SENSOR_DELAY = 1;
    private final CarStorage carStorage;
    private final Map<String, WaypointReachedCallback> waypointReachedCallbacks = new HashMap<String, WaypointReachedCallback>();
    private final Map<String, PositionChangeCallback> positionChangedCallbacks = new HashMap<String, PositionChangeCallback>();

    public CarSensor(AgentScoutEnvironment environment, Entity relatedEntity) {
        super(environment, relatedEntity);
        carStorage = environment.getCarStorage();
    }

    public void registerWaypointReachedCallback(WaypointReachedCallback waypointReachedCallback) {
        waypointReachedCallbacks.put(getRelatedEntity().getName(), waypointReachedCallback);

        carStorage.registerWaypointReachedCallback(getRelatedEntity().getName(), new WaypointReachedCallback() {

            @Override
            public void waypointReached(Point position) {
                long duration = SENSOR_DELAY;
                getEventProcessor().addEvent(TacEventType.SENSE_WAYPOINT, CarSensor.this, null, position, duration);
            }
        });
    }

    public void deregisterWaypointReachedCallback(WaypointReachedCallback waypointReachedCallback) {
        waypointReachedCallbacks.remove(getRelatedEntity().getName());

        carStorage.deregisterWaypointReachedCallback(getRelatedEntity().getName(), waypointReachedCallback);
    }

    public void registerMyPositionChangeCallback(PositionChangeCallback flyPositionChangeSensing) {
        positionChangedCallbacks.put(getRelatedEntity().getName(), flyPositionChangeSensing);

        carStorage.registerFlyPositionChangeCallback(getRelatedEntity().getName(), new PositionChangeCallback() {

            @Override
            public void sensePositionChange(Point position) {
                long duration = SENSOR_DELAY;
                getEventProcessor().addEvent(TacEventType.SENSE_POSITION, CarSensor.this, null, position, duration);
            }
        });
    }

    public Point senseMyPosition() {
        return carStorage.getPositionedEntities().get(getRelatedEntity().getName()).getPosition();
    }

    public float senseSpeed() {
        return carStorage.getCars().get(getRelatedEntity().getName()).getSpeed();
    }

    public Vector3f senseHeading() {
        return carStorage.getCars().get(getRelatedEntity().getName()).getHeading();
    }

    public Double senseDistanceToWaypoint() {
        return carStorage.getCars().get(getRelatedEntity().getName()).getDistanceToWaypoint();
    }

    @Override
    public void handleEvent(Event event) {
        if (event.isType(TacEventType.SENSE_WAYPOINT)) {
            waypointReachedCallbacks.get(getRelatedEntity().getName()).waypointReached((Point) event.getContent());
        } else if (event.isType(TacEventType.SENSE_POSITION)) {
            positionChangedCallbacks.get(getRelatedEntity().getName()).sensePositionChange((Point) event.getContent());
        } else {
            super.handleEvent(event);
        }

    }
}
