package tacv.universe.environment.storage;

import cz.agents.alite.environment.Storage;
import java.util.HashMap;
import java.util.Map;
import tacv.universe.entity.PositionedEntity;
import tacv.universe.entity.embodiment.AutomaticRegularCar;
import tacv.universe.environment.sensor.callback.PositionChangeCallback;
import tacv.universe.environment.sensor.callback.WaypointReachedCallback;
import tacv.util.Point;

public class CarStorage extends Storage implements PositionedEntityStorage {

    private final Map<String, AutomaticRegularCar> cars = new HashMap<String, AutomaticRegularCar>();
    private Map<String, WaypointReachedCallback> waypointReachedCallbacks = new HashMap<String, WaypointReachedCallback>();
    private Map<String, PositionChangeCallback> positionChangeCallbacks = new HashMap<String, PositionChangeCallback>();

    public Map<String, AutomaticRegularCar> getCars() {
        return cars;
    }

    public void addCar(AutomaticRegularCar car) {
        cars.put(car.getName(), car);
    }

    public void setNextWaypoint(String name, Point newWaypoint, Float targetVelocity, boolean isForward) {
        cars.get(name).setWaypoint(newWaypoint, targetVelocity, isForward);
    }

    public void waypointReached(String name) {
        if (waypointReachedCallbacks.containsKey(name)) {
            waypointReachedCallbacks.get(name).waypointReached(cars.get(name).getWaypoint());
        }
    }

    public void moveCar(String name, long duration, long currentTime) {
        AutomaticRegularCar car = this.cars.get(name);
        car.move(duration, currentTime);

        if (positionChangeCallbacks.containsKey(name)) {
            positionChangeCallbacks.get(name).sensePositionChange(car.getActualPosition());
        }
    }

    public void registerWaypointReachedCallback(String name, WaypointReachedCallback waypointReachedCallback) {
        waypointReachedCallbacks.put(name, waypointReachedCallback);
    }

    public void deregisterWaypointReachedCallback(String name, WaypointReachedCallback waypointReachedCallback) {
        waypointReachedCallbacks.remove(name);
    }

    public void registerFlyPositionChangeCallback(String name, PositionChangeCallback positionChangeCallback) {
        positionChangeCallbacks.put(name, positionChangeCallback);
    }

    @Override
    public Map<String, ? extends PositionedEntity> getPositionedEntities() {
        return cars;
    }
}
