package tacv.universe.map.occlusions;

import javax.vecmath.Point2d;

public class BoundingBox {
    
    private boolean initialized = false;
    private double xMin = Double.NaN;
    private double xMax = Double.NaN;
    private double yMin = Double.NaN;
    private double yMax = Double.NaN;
    
    public void reset() {
        initialized = false;
    }
    
    public void addPoint(Point2d point) {
        addPoint(point.x, point.y);
    }
    
    public void addPoint(double x, double y) {
        if (!initialized) {
            initialized = true;
            xMin = xMax = x;
            yMin = yMax = y;
            return;
        }
        if (x < xMin) {
            xMin = x;
        }
        if (x > xMax) {
            xMax = x;
        }
        if (y < yMin) {
            yMin = y;
        }
        if (y > yMax) {
            yMax = y;
        }
    }
    
    @Override
    public BoundingBox clone() {
        BoundingBox bbox = new BoundingBox();
        bbox.initialized = this.initialized;
        bbox.xMin = this.xMin;
        bbox.xMax = this.xMax;
        bbox.yMin = this.yMin;
        bbox.yMax = this.yMax;
        return bbox;
    }

    public static BoundingBox merge(BoundingBox bbox1, BoundingBox bbox2) {
        BoundingBox bbox = new BoundingBox();
        bbox.initialized = true;
        bbox.xMin = Math.min(bbox1.xMin, bbox2.xMin);
        bbox.xMax = Math.max(bbox1.xMax, bbox2.xMax);
        bbox.yMin = Math.min(bbox1.yMin, bbox2.yMin);
        bbox.yMax = Math.max(bbox1.yMax, bbox2.yMax);
        return bbox;
    }

    public boolean isPointInside(Point2d point) {
        return point.x >= xMin && point.x <= xMax &&
               point.y >= yMin && point.y <= yMax;
    }
    
    public double getXMin() {
        return xMin;
    }
    
    public double getXMax() {
        return xMax;
    }
    
    public double getYMin() {
        return yMin;
    }
    
    public double getYMax() {
        return yMax;
    }
}
