package tacv.util;

import cz.agents.alite.planner.spatialmaneuver.zone.*;
import cz.agents.alite.vis.element.Line;
import cz.agents.alite.vis.element.implemetation.LineImpl;
import java.util.List;
import javax.vecmath.Point2d;
import tacv.util.Point;
import tacv.util.Vector;

/**
 *
 * @author Jiri Vokrinek
 */
public class LineZoneVisitor implements ZoneVisitor {
    private final List<Line> lines;
    private final Vector translation = new Vector();

    public LineZoneVisitor(List<Line> lines) {
        this.lines = lines;
    }

    @Override
    public void visit(CylinderZone zone) {
        Point2d oldPoint = null;
        for (double angle = 0; angle <= 2.1 * Math.PI; angle += Math.PI / 32.0) {
            Point2d point = new Point2d(zone.getSemiAxes().x * Math.cos(angle), zone.getSemiAxes().y * Math.sin(angle));
            if (oldPoint != null) {
                lines.add(new LineImpl(new Point(translation.x + oldPoint.x, translation.y + oldPoint.y, translation.z), new Point(translation.x + point.x, translation.y + point.y, translation.z)));
            }
            oldPoint = point;
        }
    }

    @Override
    public void visit(EmptyZone zone) {
    }

    @Override
    public void visit(GroupZone zone) {
        for (Zone subZone : zone.getSubZones()) {
            subZone.accept(this);
        }
    }

    @Override
    public void visit(TransformZone zone) {
        if (zone.getAngleRotation() != 0 || zone.getSize().x != 1 || zone.getSize().y != 1) {
            throw new RuntimeException("Not implemented!");
        }
        translation.add(zone.getTranslation());
        zone.getZone().accept(this);
        translation.sub(zone.getTranslation());
    }

    @Override
    public void visit(EllipsoidZone zone) {
        throw new RuntimeException("Not implemented!");
    }

    @Override
    public void visit(BoxZone zone) {
        throw new RuntimeException("Not implemented!");
    }

    @Override
    public void visit(PrismZone zone) {
        Point2d oldPoint = null;
        Point2d firstPoint = null;
        for (Point2d point : zone.getPoints()) {
            if (oldPoint != null) {
                lines.add(new LineImpl(new Point(translation.x + oldPoint.x, translation.y + oldPoint.y, translation.z), new Point(translation.x + point.x, translation.y + point.y, translation.z)));
            } else {
                firstPoint = point;
            }
            oldPoint = point;
        }
        lines.add(new LineImpl(new Point(translation.x + oldPoint.x, translation.y + oldPoint.y, translation.z), new Point(translation.x + firstPoint.x, translation.y + firstPoint.y, translation.z)));
    }
    
}
