package tacv.vis3d.spatial;

import com.jme3.asset.AssetManager;
import com.jme3.bounding.BoundingBox;
import com.jme3.material.Material;
import com.jme3.math.*;
import com.jme3.scene.Geometry;
import com.jme3.scene.Node;
import com.jme3.scene.Spatial;
import com.jme3.scene.control.Control;
import com.jme3.scene.shape.Box;
import com.jme3.scene.shape.Cylinder;
import cz.agents.alite.jbullet.RegularCarModel;
import cz.agents.alite.vis3d.Vis3d;

public final class MdarsProvider {

    private static final boolean DEBUG_BULLET_SHAPES = false;
    private static final float MODEL_SCALE = 1f;
    private static final Quaternion MODEL_LOAD_ROTATION = new Quaternion().fromAngles(0, 0, 0);
    private static final String MDARS_BODY_LOAD = "models/mdars/mdars.mesh.xml";
//	private static final String MDARS_BODY_LOAD = "models/mdars/mdars.obj";
    private static final String MDARS_WHEEL_LOAD = "models/mdars/mdars_wheel.mesh.xml";
//	private static final String MDARS_WHEEL_LOAD = "models/mdars/mdars_wheel.obj";
    private static MdarsProvider instance;
    private Mdars referentialInstance;
    Vis3d vis;

    private MdarsProvider(Vis3d vis) {
        this.vis = vis;
        this.init();
    }

    public void init() {
        AssetManager assetManager = vis.getAssetManager();
        referentialInstance = new Mdars("referentialMdar");
//		Node wheels = new Node("wheels");
        Spatial body = assetManager.loadModel(MDARS_BODY_LOAD);
        body.setLocalTranslation(new Vector3f(0, 0.2f, 0));
        referentialInstance.attachChild(body);
        body.rotate(MODEL_LOAD_ROTATION);

        Spatial wheelModel = assetManager.loadModel(MDARS_WHEEL_LOAD);//mesh.xml");
//		wheelModel.setMaterial(MaterialProvider.getInstance(vis).getTransparentMaterial(1, 1, 1, 0.01f));

//		wheelModel.setShadowMode(ShadowMode.Receive);
//		wheelModel.rotate(MODEL_LOAD_ROTATION);

        Node wheelLB = new Node("LB");
        wheelLB.attachChild(wheelModel.deepClone());
        wheelLB.setLocalTranslation(-.7f, .34F, -.9f);
        referentialInstance.attachChild(wheelLB);

        Node wheelRB = new Node("RB");
        wheelRB.attachChild(wheelModel.deepClone());
        wheelRB.setLocalTranslation(.7f, .34F, -.9f);
        referentialInstance.attachChild(wheelRB);

        Node wheelLF = new Node("LF");
        wheelLF.attachChild(wheelModel.deepClone());
        wheelLF.setLocalTranslation(-.7f, .34F, .9f);
        referentialInstance.attachChild(wheelLF);

        Node wheelRF = new Node("RF");
        wheelRF.attachChild(wheelModel.deepClone());
        wheelRF.setLocalTranslation(.7f, .34F, .9f);
        referentialInstance.attachChild(wheelRF);

        if (DEBUG_BULLET_SHAPES) {
            Box box = new Box(Vector3f.ZERO,
                    0.5f * RegularCarModel.carDefinition.getCollisionBox().x,
                    0.5f * RegularCarModel.carDefinition.getCollisionBox().z,
                    0.5f * RegularCarModel.carDefinition.getCollisionBox().y);
            Geometry boxGeometry = new Geometry("Box", box);
            Material boxMaterial = new Material(assetManager, "Common/MatDefs/Misc/Unshaded.j3md");
            boxMaterial.setColor("Color", ColorRGBA.Blue);
            boxGeometry.setMaterial(boxMaterial);
            referentialInstance.attachChild(boxGeometry);

            float radius = RegularCarModel.carDefinition.getWheelDefinitions()[0].radius;
            Cylinder cylinder = new Cylinder(4, 10, radius, radius, true);
            Geometry cylinderGeometry = new Geometry("Cylinder", cylinder);
            Matrix3f rotation = new Matrix3f();
            rotation.fromAngleAxis(FastMath.PI * 0.5f, Vector3f.UNIT_Y);
            cylinderGeometry.setLocalRotation(rotation);
            Material cylinderMaterial = new Material(assetManager, "Common/MatDefs/Misc/Unshaded.j3md");
            cylinderMaterial.setColor("Color", ColorRGBA.Red);
            cylinderGeometry.setMaterial(cylinderMaterial);
            wheelLF.attachChild(cylinderGeometry.deepClone());
            wheelRF.attachChild(cylinderGeometry.deepClone());
            wheelLB.attachChild(cylinderGeometry.deepClone());
            wheelRB.attachChild(cylinderGeometry.deepClone());
        }

        referentialInstance.setModelBound(new BoundingBox());
        referentialInstance.updateModelBound();

        referentialInstance.setLocalScale(MODEL_SCALE);
    }

    public static MdarsProvider getInstance(Vis3d vis) {
        if(instance == null || instance.vis == null || !vis.equals(instance.vis)){
            instance = new MdarsProvider(vis);
        }
        return instance;
    }

    public Spatial getNext(String name, Control... ctrl) {
        Node spatial = (Node) referentialInstance.deepClone();
        spatial.setName(name);
        for (Control c : ctrl) {
            spatial.addControl(c);
        }
        return spatial;
    }
}
